/*
	Copyright (C) 2001, 2002 Artem V. Andreev (artem@AA5779.spb.edu)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#ifdef HAVE_LIBREADLINE
#include <readline/readline.h>
#include <readline/history.h>
#include <modules.h>
#include <util.h>
#include <dstring.h>
#include "version.h"
/// \section[NAME]
// \em[readline] -- a module for Tensile (see \ref[sl(1)])

/// \section[OVERVIEW]
// This module replaces the standard Tensile \em[readline] with a binding
// to GNU \ref[readline(3)]. The interface is unaffected.
//
// Non-empty input strings are added to history
// 
// See \em[info readline] on more details
// \note extended facilities such as custom readline functions, as
// well as history manipulating functions are not implemented


static module *this_mod;
extern procstack *get_pstack(void);

void readline_LTX_init(module *this)
{
	this_mod = this;
	using_history();
	read_history(NULL);
}

void readline_LTX_finish(module *this)
{
	write_history(NULL);
}

static void bi_readline(chunk *current, interpreter *i, value *args)
{
	char *prompt = args ? DS_BODY(args->val) : "";
	char *in = readline(prompt);
	if(in)
	{
		if(*in)
			add_history(in);
		ds_appendstr(get_pstack()->state->buffer, in);
		free(in);
	}
}

static master_instruction_src readline_src[] = {{"readline", 1, bi_readline, 1, 1, 1},
										 {NULL}
										};

void readline_LTX_attach(mod_attach *ma)
{
	ma->data = xmalloc(sizeof(readline_src) * sizeof(instruction_t) / sizeof(*readline_src));
	replace_builtins(ma->i, readline_src, ma->data);
}

const char readline_LTX_version[] = VERSION;
const char readline_LTX_date[] = __DATE__;

#endif
/// \section[AUTHOR]
// Artem V. Andreev

/// \section[COPYRIGHT]
// see \ref[sl(1)]

/// \trailer
